package com.ebay.redlasersdk;

import android.app.Activity;
import android.graphics.Rect;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import com.mirasense.scanditsdk.ScanditSDKBarcodePicker;
import com.mirasense.scanditsdk.ScanditSDKScanSettings;
import com.mirasense.scanditsdk.interfaces.ScanditSDK;
import com.mirasense.scanditsdk.interfaces.ScanditSDKCode;
import com.mirasense.scanditsdk.interfaces.ScanditSDKOnScanListener;
import com.mirasense.scanditsdk.interfaces.ScanditSDKScanSession;
import com.scandit.base.camera.SbICamera;
import com.scandit.recognition.Barcode;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

/**
 * Created by mo on 21/08/15.
 */
public abstract class BarcodeScanActivity extends Activity implements ScanditSDKOnScanListener {

    public BarcodeTypes enabledTypes = new BarcodeTypes();

    protected ScanditSDKBarcodePicker mBarcodePicker = null;

    public static final String PREF_ORIENTATION_LANDSCAPE = "orientationLandscape";
    public static final String PREF_ORIENTATION_PORTRAIT = "orientationPortrait";
    public static final String REDLASER_CAMERA_PREVIEW_VIEW_TAG = "RedLaser Camera Preview Tag";
    public static final String RESULT_EXTRA = "com.ebay.redlasersdk.results";

    private int mRequestedCameraIndex = 0;

    private static final int SEND_UPDATE = 1;
    private Handler sHandler = new Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case SEND_UPDATE:
                    onScanStatusUpdate((Map<String, Object>) msg.obj);
                    break;
            }
        }
    };


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        enabledTypes.setCallback(this);
    }

    @Override
    protected void onResume() {
        super.onResume();

        if (mBarcodePicker == null) {
            mBarcodePicker = new ScanditSDKBarcodePicker(this, scanditAppKey(), createSettings());
            mBarcodePicker.addOnScanListener(this);

            if (getPreviewView() == null) {
                ViewGroup group = (ViewGroup) findViewById(android.R.id.content);
                group.addView(mBarcodePicker, 0);
            } else {
                getPreviewView().addView(mBarcodePicker);
            }
        }

        mBarcodePicker.startScanning();
    }

    @Override
    protected void onPause() {
        super.onPause();

        mBarcodePicker.stopScanning();
    }

    protected abstract String scanditAppKey();

    @Override
    public void didScan(ScanditSDKScanSession session) {
        HashSet<BarcodeResult> barcodes = hashSetFromCodeList(session.getAllCodes());
        HashSet<BarcodeResult> newBarcodes = hashSetFromCodeList(session.getNewlyDecodedCodes());

        HashMap<String, Object> scanStatus = new HashMap<String, Object>();
        scanStatus.put(Status.STATUS_FOUND_BARCODES, barcodes);
        scanStatus.put(Status.STATUS_NEW_FOUND_BARCODES, newBarcodes);
        scanStatus.put(Status.STATUS_CAMERA_SNAPSHOT, null);
        scanStatus.put(Status.STATUS_GUIDANCE, 0);
        scanStatus.put(Status.STATUS_IN_RANGE, false);
        scanStatus.put(Status.STATUS_PARTIAL_BARCODE, null);

        sHandler.sendMessage(sHandler.obtainMessage(SEND_UPDATE, scanStatus));
    }

    private HashSet<BarcodeResult> hashSetFromCodeList(List<ScanditSDKCode> codes) {
        HashSet<BarcodeResult> results = new HashSet<BarcodeResult>();
        for (ScanditSDKCode code : codes) {
            BarcodeResult result = new BarcodeResult();
            result.barcodeString = code.getData();
            result.extendedBarcodeString = code.getData();

            if (code.getSymbologyString().equals("EAN13") || code.getSymbologyString().equals("UPC12")) {
                result.barcodeType = BarcodeTypes.EAN13;
            } else if (code.getSymbologyString().equals("UPCE")) {
                result.barcodeType = BarcodeTypes.UPCE;
            } else if (code.getSymbologyString().equals("EAN8")) {
                result.barcodeType = BarcodeTypes.EAN8;
            } else if (code.getSymbologyString().equals("QR")) {
                result.barcodeType = BarcodeTypes.QRCODE;
            } else if (code.getSymbologyString().equals("CODE128")) {
                result.barcodeType = BarcodeTypes.CODE128;
            } else if (code.getSymbologyString().equals("CODE39")) {
                result.barcodeType = BarcodeTypes.CODE39;
            } else if (code.getSymbologyString().equals("DATAMATRIX")) {
                result.barcodeType = BarcodeTypes.DATAMATRIX;
            } else if (code.getSymbologyString().equals("ITF")) {
                result.barcodeType = BarcodeTypes.ITF;
            } else if (code.getSymbologyString().equals("CODE93")) {
                result.barcodeType = BarcodeTypes.CODE93;
            } else if (code.getSymbologyString().equals("GS1-DATABAR")) {
                result.barcodeType = BarcodeTypes.GS1DATABAR;
            } else if (code.getSymbologyString().equals("CODABAR")) {
                result.barcodeType = BarcodeTypes.CODABAR;
            } else if (code.getSymbologyString().equals("FIVE-DIGIT-ADD-ON")) {
                result.barcodeType = BarcodeTypes.EAN5;
            } else if (code.getSymbologyString().equals("TWO-DIGIT-ADD-ON")) {
                result.barcodeType = BarcodeTypes.EAN2;
            } else if (code.getSymbologyString().equals("PDF417")) {
                result.barcodeType = BarcodeTypes.PDF417;
            } else if (code.getSymbologyString().equals("GS1-DATABAR-EXPANDED")) {
                result.barcodeType = BarcodeTypes.GS1DATABAR_EXPANDED;
            }

            results.add(result);
        }

        return results;
    }

    protected void doneScanning() {
        mBarcodePicker.stopScanning();
        finish();
    }

    public String getOrientationSetting() {
        return PREF_ORIENTATION_PORTRAIT;
    }

    protected FrameLayout getPreviewView() {
        return null;
    }

    protected boolean getTorch() {
        return false;
    }

    protected boolean isTorchAvailable() {
        return true;
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();

        doneScanning();
    }

    public void onCameraError(int error) {

    }

    public void onCameraPreviewStarted() {

    }

    protected void onError(int error) {

    }

    protected abstract void onScanStatusUpdate(Map<String, Object> scanStatus);

    protected void requestCameraIndex(int index) {
        mRequestedCameraIndex = index;
    }

    protected void requestImageData() {

    }

    protected void setActiveRect(Rect activeRect) {

    }

    protected void setTorch(boolean torchValue) {
        mBarcodePicker.switchTorchOn(torchValue);
    }

    protected void enabledBarcodeTypesChanged() {

    }

    protected void barcodeTypesDidUpdate(int symbology, boolean enabled) {
        enabledBarcodeTypesChanged();
        if (mBarcodePicker != null) {
            switch (symbology) {
                case BarcodeTypes.EAN13:
                    mBarcodePicker.setEan13AndUpc12Enabled(enabled);
                    break;

                case BarcodeTypes.UPCE:
                    mBarcodePicker.setUpceEnabled(enabled);
                    break;

                case BarcodeTypes.EAN8:
                    mBarcodePicker.setEan8Enabled(enabled);
                    break;

                case BarcodeTypes.QRCODE:
                    mBarcodePicker.setQrEnabled(enabled);
                    break;

                case BarcodeTypes.CODE128:
                    mBarcodePicker.setCode128Enabled(enabled);
                    break;

                case BarcodeTypes.CODE39:
                    mBarcodePicker.setCode39Enabled(enabled);
                    break;

                case BarcodeTypes.DATAMATRIX:
                    mBarcodePicker.setDataMatrixEnabled(enabled);
                    break;

                case BarcodeTypes.ITF:
                    mBarcodePicker.setItfEnabled(enabled);
                    break;

                case BarcodeTypes.CODE93:
                    mBarcodePicker.setCode93Enabled(enabled);
                    break;

                case BarcodeTypes.GS1DATABAR:
                    mBarcodePicker.setGS1DataBarEnabled(enabled);
                    break;

                case BarcodeTypes.CODABAR:
                    mBarcodePicker.setCodabarEnabled(enabled);
                    break;

                case BarcodeTypes.EAN5:
                    mBarcodePicker.setFiveDigitAddOnEnabled(enabled);
                    break;

                case BarcodeTypes.EAN2:
                    mBarcodePicker.setTwoDigitAddOnEnabled(enabled);
                    break;

                case BarcodeTypes.PDF417:
                    mBarcodePicker.setPdf417Enabled(enabled);
                    break;

                case BarcodeTypes.GS1DATABAR_EXPANDED:
                    mBarcodePicker.setGS1DataBarExpandedEnabled(enabled);
                    break;
            }

            if (enabledTypes.getEAN5() || enabledTypes.getEAN2()) {
                mBarcodePicker.setMaxNumCodesPerFrame(2);
            } else {
                mBarcodePicker.setMaxNumCodesPerFrame(1);
            }
        }


    }

    public ScanditSDKScanSettings createSettings() {
        ScanditSDKScanSettings settings = ScanditSDKScanSettings.getDefaultSettings();

        if (enabledTypes.getEan13()) {
            settings.enableSymbology(ScanditSDK.Symbology.EAN13);
            settings.enableSymbology(ScanditSDK.Symbology.UPC12);
        }

        if (enabledTypes.getUpce()) {
            settings.enableSymbology(ScanditSDK.Symbology.UPCE);
        }

        if (enabledTypes.getEan8()) {
            settings.enableSymbology(ScanditSDK.Symbology.EAN8);
        }

        if (enabledTypes.getQRCode()) {
            settings.enableSymbology(ScanditSDK.Symbology.QR);
        }

        if (enabledTypes.getCode128()) {
            settings.enableSymbology(ScanditSDK.Symbology.CODE128);
        }

        if (enabledTypes.getCode39()) {
            settings.enableSymbology(ScanditSDK.Symbology.CODE39);
        }

        if (enabledTypes.getDataMatrix()) {
            settings.enableSymbology(ScanditSDK.Symbology.DATAMATRIX);
        }

        if (enabledTypes.getITF()) {
            settings.enableSymbology(ScanditSDK.Symbology.ITF);
        }

        if (enabledTypes.getCode93()) {
            settings.enableSymbology(ScanditSDK.Symbology.CODE93);
        }

        if (enabledTypes.getGS1Databar()) {
            settings.enableSymbology(ScanditSDK.Symbology.GS1_DATABAR);
        }

        if (enabledTypes.getCodabar()) {
            settings.enableSymbology(ScanditSDK.Symbology.CODABAR);
        }

        if (enabledTypes.getEAN5()) {
            settings.enableSymbology(ScanditSDK.Symbology.FIVE_DIGIT_ADD_ON);
            settings.setMaxNumCodesPerFrame(2);
        }

        if (enabledTypes.getEAN2()) {
            settings.enableSymbology(ScanditSDK.Symbology.TWO_DIGIT_ADD_ON);
            settings.setMaxNumCodesPerFrame(2);
        }

        if (enabledTypes.getPDF417()) {
            settings.enableSymbology(ScanditSDK.Symbology.PDF417);
        }

        if (enabledTypes.getGS1DatabarExpanded()) {
            settings.enableSymbology(ScanditSDK.Symbology.GS1_DATABAR_EXPANDED);
        }

        settings.setCameraFacingPreference(mRequestedCameraIndex == 0 ? ScanditSDK.CAMERA_FACING_BACK : ScanditSDK.CAMERA_FACING_FRONT);

        return settings;
    }


    public static class Status {
        public static final String STATUS_CAMERA_SNAPSHOT = "CameraSnapshot";
        public static final String STATUS_FOUND_BARCODES = "FoundBarcodes";
        public static final String STATUS_GUIDANCE = "Guidance";
        public static final String STATUS_IN_RANGE = "InRange";
        public static final String STATUS_NEW_FOUND_BARCODES = "NewFoundBarcodes";
        public static final String STATUS_PARTIAL_BARCODE = "PartialBarcode";
    }
}
